
# ================================================================================================== #
#                                                                                                    #
#                            Introductory Tutorials for Scientific Python                            #
#                  ===============================================================                   #
#                          SAMSI/NCSU Undergraduate Workshop: May 15, 2017                           #
#                                                                                                    #
#                                          Ahmed Attia                                               #
#                                North Carolina State University,                                    #
#                      Statistical and Applied Mathematical Science Institute                        #
#                                  attia@{vt.edu; samsi.info}                                        #
#                               http://people.cs.vt.edu/~attia/                                      #
#                                                                                                    #
# ================================================================================================== # 
#   Lesson 3:                                                                                        #
#             - Data Structures, and iterables: Lists.                                               #
# ================================================================================================== # 


# You should be able to understand how the following is created by now!
msg = "Python iterables refer to any object capable of returning its members one at a time."
msg += "Basic examples include:\n\t  \r- lists, tuples, dictionaries, and strings!"
print(msg)
# raw_input("\npress Enter to continue")

# 1- Lists
# -------------
msg = """
A list is an ordered, mutable object, that contains a set of (non)homogenous elements.
It is created using square brackets, with entries separated by commas:
e.g.
    list_0 = [1, 2, 12, 12.44, -10000]
    list_1 = [5, 'SAMSI', None, "", 30%7]
    list_2 = [3>-1+5, [23, 12, -50], list_1]
"""
print(msg)
raw_input("\npress Enter to continue")

list_0 = [1, 2, 12, 12.44, -10000]
print("print(list_0) -> "), 
print(list_0)

list_1 = [5, 'SAMSI', None, "", 30%7]
print("print(list_1) -> "), 
print(list_1)

list_2 = [3>-1+5, [23, 12, -50], list_1]
print("print(list_2) -> "), 
print(list_2)
raw_input("\npress Enter to continue")

msg = """
    'lists' are indexed objects, with index starting at 0
    For example, to access the third entry of the list objet 'list_0', you can use:
    list_0[2]
"""
print(msg)
raw_input("\npress Enter to continue")

# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 0:                                              #
#                  add 300 to the second entry of 'list_0', then print the list                      #
# -------------------------------------------------------------------------------------------------- #
list_0[1] += 300
print("Adding 300 to the second entry of list_0; updated list:"),
print(list_0)
raw_input("\npress Enter to continue")

# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 1:                                              #
#                  add 300 to the second entry of 'list_2', then print the list                      #
# -------------------------------------------------------------------------------------------------- #
list_2[1][0] += 300
list_2[1][1] += 300
list_2[1][2] += 300
# list_2[1] = [ val+300 for val in list_2[1] ]  # we should be able to do better with this!
print("Adding 300 to the elements of the second entry of list_2; updated list:"),
print(list_2)
raw_input("\npress Enter to continue")

msg = """
You can manipulate a list using many associated methods, including:
append()  : L.append(object) -- append object to end
pop()     : L.pop([index]) -> item -- remove and return item at index (default last)
sort()    : L.sort(cmp=None, key=None, reverse=False) -- stable sort *IN PLACE*
reverse() : L.reverse() -- reverse *IN PLACE*
...
"""
print(msg)
raw_input("\npress Enter to continue")

# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 2:                                              #
#                  reverse 'list_2', then remove the last entry of the second element,               #
#                  and then add 14 to the end of 'list_2'                                            #
# -------------------------------------------------------------------------------------------------- #


msg = """ (SLICING)
Slicing in general means getting a slice of an indexed iterable.
You can use ':' to access a range of entries in an indexed iterable (e.g. a list, tuple, etc)
The indexing in Python starts at zero rather than at 1, where the indexing can be done from the end
    of the iterable starting at -1.

For example:
-------------
list_0[1:3]  # accesses the entries in list_0 starting at index 1, and ending just BEFORE index 3
list_0[-1]  # accesses the last entry of list_0
list_0[0:-1]  # accesses the entries in list_0 starting at index 0, and ending just BEFORE last entry
list_0[:-1]  # equivalent to previous statement
list0[1:]  # accesses the entries in list_0 starting at index 1, upto last entry of the list
etc.
"""
print(msg)
raw_input("\npress Enter to continue")


# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 3:                                              #
#                  concatenate the two lists 'list_0', and 'list_1' in a new list 'list_3',          #
#                  then print it's contents                                                          #
# -------------------------------------------------------------------------------------------------- #
list_3 = list_0 + list_1
print(list_3)
raw_input("\npress Enter to continue")

msg = """
    So, you can add two (or more) lists.
    Like strings, right?
    You can alternatively use 'extend' method associated with a list object!
    
    You can also multiply a list by an integer to replicate its entries as in strings!
    
"""
print(msg)
print("list_0 * 3 -> "),
print(list_0 * 3)

# ================================================================================================== # 

