
# ================================================================================================== #
#                                                                                                    #
#                            Introductory Tutorials for Scientific Python                            #
#                  ===============================================================                   #
#                          SAMSI/NCSU Undergraduate Workshop: May 15, 2017                           #
#                                                                                                    #
#                                          Ahmed Attia                                               #
#                                North Carolina State University,                                    #
#                      Statistical and Applied Mathematical Science Institute                        #
#                                  attia@{vt.edu; samsi.info}                                        #
#                               http://people.cs.vt.edu/~attia/                                      #
#                                                                                                    #
# ================================================================================================== # 
#   Lesson 4:                                                                                        #
#             - Branching and Looping: Loops                                                         #
# ================================================================================================== # 

loop_structure = """
You can design loops (executing statements repeatedly) in Python using:
1- for,
2- while

In Python, 'for' loop is built to iterate over an iterable!

For loop takes the form (Notice the indentation of the ``loop body" block:
----------------------------------------------------------------------------

for [index] in [iterable]:
    <one or more indented statements>
    <break>
    <continue>
    
While loop takes the form (Notice the indentation of the ``loop body" block:
----------------------------------------------------------------------------

while [boolean expression]:
    <one or more indented statements>
    <break>
    <continue>

%s\n\t\tSee the INDENTATION!\n%s
""" % ('*'*55, '*'*55)
print(loop_structure)
raw_input("\npress any key to continue")


#
# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 0:                                              #
#                            Loop over, and print numbers from 0 to 11:                              #
# -------------------------------------------------------------------------------------------------- #
sep = '-'*45  # what do you think this is ?
print("%s\nExercise: Use a for loop to print numbers from 0 to 11. \n%s" % (sep*2, sep*2))
#

# Method 0:
#-----------------
print("\nMethod 0: create a list, then iterate over it:\n %s" % sep )
vals = range(12)  # create a list of integers from 0 to 11
for val in vals:  # iterate over the values in the list vals
    print("Value: %d" % val)

raw_input("\npress any key to continue")

# Method 1:
#-----------------
print("\nMethod 1: create a list, then iterate over it:\n %s" % sep )
# This can be combined as follows
for val in range(12):
    print("Value: %d" % val )

raw_input("\npress any key to continue")

# Method 2:
#-----------------
print("\nA better approach is as follows:\n %s" % sep )
for val in xrange(12):
    print("Value: %d" % val )


raw_input("\npress any key to continue")

# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 1:                                              #
#               Create a dictionary that contains work days, and a discription of each one           #
#               then of course print that out!
# -------------------------------------------------------------------------------------------------- #
msg = "Create a dictionary that contains work days, and a discription of each one,\n   \
       \rthen of course print that out!"
print("%s\nExercise: %s. \n%s" % (sep*2, msg, sep*2))

raw_input("\npress any key to continue")

# Method 0:
#-----------------
print("\nMethod 0: create a dictionary, then iterate over it's keys:\n %s" % sep )
# Create the dictionary:
week_days = dict(Tuesday="boring", 
                 Wednesday="Boring",
                 Thursday="BORING",
                 Friday="Let's do some work before the weekend!"
                 )
                 
# iterate over the dictionary keys;
for day in week_days:
    print(day + ": " + week_days[day])
                 
raw_input("\npress any key to continue")

# Method 1:
#-----------------
# Create two lists (of equal lengths):
print("\nMethod 1: create TWO disjoint lists of equal lengths,\n \
       \rthen iterate over them concurrently:\n %s" % sep )
#       
days = ["Tuesday", "Wednesday", "Thursday", "Friday"]
descriptions = ["boring", "Boring", "BORING", "Let's do some work before the weekend!"]
 
# iterate over the two lists concurrently;
for day, description in zip(days, descriptions):
   print(day + ": " + description)

raw_input("\npress any key to continue")
 
msg = """
# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 2:                                              #
#       Create a list of Fibonacci numbers of length 50, and print them indexed starting at 1,       #
#       i.e. the output should be for example on the form:                                           #
#       Fib #1 is: 1                                                                                 #
#       Fib #2 is: 1                                                                                 #
#       Fib #3 is: 2                                                                                 #
#       Fib #4 is: 3                                                                                 #
#       Fib #5 is: 5                                                                                 #
#       ...                                                                                          #
# -------------------------------------------------------------------------------------------------- #
"""
# An example:
# Gradually, create the list
print(msg)

fib_nums = [1, 1]
for ind in xrange(2, 50):
    fib_nums.append(fib_nums[ind-2] + fib_nums[ind-1])

# output the entries of the constructed list:
print("Solution:\n%s" % sep)
for list_ind in xrange(len(fib_nums)):
    print("Fib #%2d is: %d" %(list_ind+1, fib_nums[list_ind]))

raw_input("\npress any key to continue")

msg = """
# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 3:                                              #
#       Create a for loop to print each each character of the following statement on a line          #
#       "This is reALLy Becoming A BORING daY!"                                                      #
#       with a yes/no answer that indicates wheather it is lower case or not!                        #
#       Hint: the method lower() associated with string objects may be useful!                       #
# -------------------------------------------------------------------------------------------------- #
"""
print(msg)

# Sol:
msg = "This is reALLy Becoming A BORING daY!"
for char in msg:
    dec = 'Lower Case' if char.lower() == char else 'Upper Case'
    print(" '%s' is: %s" % (char, dec) )

raw_input("\npress any key to continue")

msg = """
# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 4:                                              #
#       Create a list of integers between 3, 10 raised to power 5, then print it                     #
# -------------------------------------------------------------------------------------------------- #
"""
# An example:
print(msg)
num_powers = [num**5 for num in xrange(3, 11)]
print("A list of integers between 3, 10 raised to power 5:")
for num, power in zip(xrange(3, 11), num_powers):
    print("%2d ** 5 = %d" % (num, power))
    
msg = """
  An advanced approach to handle strings, and patterns is to use Python regular expression module 're'
  See: https://docs.python.org/2/library/re.html
"""
raw_input("\npress any key to continue")

#
msg = """
# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 5:                                              #
#               Print Fibonacci numbers less than 10**8.                                             #
#               For memory efficiency, DONOT keep the whole list in memory!                          #
#               Hints: 1- both 'break, 'continue' are usefule statements to be used inside a loop,   #
#                      2- would it make more sense to use a 'while' loop?                            #
# -------------------------------------------------------------------------------------------------- #
"""
print(msg)
raw_input("\npress any key to continue")

# One way is as follows:
fib_nums = [1, 1]
print("Fib # 1 is: " + str(fib_nums[0]))
print("Fib # 2 is: " + str(fib_nums[1]))
counter = 2
while True:
    new_num = fib_nums[0] + fib_nums[1]
    if new_num >= 10**8:
        print("Nothing more to print. Terminating...")
        break 
    else:  # the following block can be pulled of else branch, right? Why?
        counter += 1
        fib_nums[0] = fib_nums[1]
        fib_nums[1] = new_num
        print("Fib #%2d is: %d" %(counter, fib_nums[1]))
        
# ================================================================================================== # 

