
# ================================================================================================== #
#                                                                                                    #
#                            Introductory Tutorials for Scientific Python                            #
#                  ===============================================================                   #
#                          SAMSI/NCSU Undergraduate Workshop: May 15, 2017                           #
#                                                                                                    #
#                                          Ahmed Attia                                               #
#                                North Carolina State University,                                    #
#                      Statistical and Applied Mathematical Science Institute                        #
#                                  attia@{vt.edu; samsi.info}                                        #
#                               http://people.cs.vt.edu/~attia/                                      #
#                                                                                                    #
# ================================================================================================== # 
#   Lesson 7:                                                                                        #
#             - Introduction to Numpy.                                                               #
# ================================================================================================== # 

msg = """
# ================================================================================================== #
#              Numpy is the most popular package for Numerical calculations in Python                #
# ================================================================================================== #
#                                                                                                    #
# At the heart of Numpy is the consept of nd-array (N-Dimensional array) which gives access to       #
# Matrix linear algebra and related mathematical functionalities.                                    #
#                                                                                                    #
# It is very common to import the numpy module as follows:                                           #
# import numpy as np                                                                                 #
#                                                                                                    #
# 1- To create a numpy array, use the method np.array()                                              #
#    This method can take as input a (nested) sequence of entries, and returns a numpy array object. #
#                                                                                                    #
# 2- A nested sequence, e.g. a list of lists, is used if a multidimensional array is desired.        #
#                                                                                                    #
# ================================================================================================== #
#                 * A commonly used, yet different, method is np.asarray() *                         #
# ================================================================================================== #
# Good Resource: https://docs.scipy.org/doc/numpy-dev/user/quickstart.html                           #
# ================================================================================================== #
"""
print(msg)
raw_input("press Enter to continue")

# Creating an array from an iterable:
example_0 = """
Example 0:
-----------------
    Create a one dimensional array named array_0 with entries 12, 13.0, -22, 15,
    and then check the method associated with that array
    Hint: use the dot operator in IPython
    
    Note, that you have to import Numpy first, otherwise you will get a 'NameError' exception!
"""
import numpy as np
print(example_0)
array_0 = np.array([12, 13.0, -22, 15])
array_0 = np.array((12, 13.0, -22, 15))  # Another way
print(array_0)

raw_input("\npress Enter to continue")
# Creating a 2-D array from a nested list
example_1 = """
    Example 1:
    -----------------
        Create a two dimensional array named array_1 of size 3 x 5 with all entries set to 1,
        with an integer data type.
"""
print(example_1)
# array of 3 rows, each containing 5 entries
array_1 = np.array([[1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1]])  

raw_input("press Enter to continue")

remark = """
Python figures out automatically the appropriate data type of the array elements, 
however, this can be specified using the optional argument 'dtype' of the method array()
"""
print(remark)


slicing_msg = """
The contents of a numpy nd-array object can be accessed by slicing
To slice the second to last row, and third to to fourth column of array_1:
array_1_slice = array_1[1:, 2:4]  
"""
array_1_slice = array_1[1:, 2:4]  # slices second to last row, and third to to fourth column!
print(array_1_slice)

raw_input("press Enter to continue")
# Exercise 0:
exercise0 = """
# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 0:                                              #
#                                                                                                    #
# 1- Create a one dimensional float numpy array named array_2 containing numbers                     #
#    ranging from 3 to 55, with step 2 efficiently, and print it                                     #
#                                                                                                    #
# -------------------------------------------------------------------------------------------------- #
"""
print(exercise0)

# Solution:
array_2 = np.array(xrange(3, 56, 2), dtype=float)
print(array_2)


raw_input("press Enter to continue")
remark = """
Numpy nd object has tons of associated attributes, and methods including:

        _____________________________________________________________________________________________
         Attribute   | Action                                                                      
        _____________________________________________________________________________________________
         ndim        | 
         shape       | 
         size        | 
         dtype       | 
         ...
        _____________________________________________________________________________________________
        
        
        _____________________________________________________________________________________________
         Method      | Action                                                                      
        _____________________________________________________________________________________________
         copy()      | 
         dot()       | 
         min()       | 
         max()       | 
         mean()      | 
         std()       | 
         conj()      | 
         diagonal()  |
         ...
        _____________________________________________________________________________________________

"""
print(remark)


raw_input("press Enter to continue")
np_remark = """
Numpy offers a wide range of functions, and submodules that gives access to all basic 
linear algebra and statitics. 

 Array Functions:
 ==============================
    Widely used functions include:
    
        _____________________________________________________________________________________________
         Method              | Action
        _____________________________________________________________________________________________
         eye()               |
         zeros()             |
         zeros_like()        |
         ones()              |
         ones_like()         |
         empty()             |
         empty_like()        |
         arange()            |
         linspace()          |
         diagonal()          |
         diag()              | 
         ...
        _____________________________________________________________________________________________
         reshape()           | 
         squeeze()           | 
         ...
        _____________________________________________________________________________________________
         add()               | 
         dot()               | 
         multiply()          | 
         ...
        _____________________________________________________________________________________________
         round()             | 
         araound()           |
         floor()             |
         ...
        _____________________________________________________________________________________________
         min()               |
         max()               |
         mean()              | 
         std()               | 
         ...
        _____________________________________________________________________________________________
         square()            | 
         pow()               | 
         ...
        _____________________________________________________________________________________________
         sin()               |
         cos()               |
         arcsin()            |
         arccos()            |
         ...                 
         tanh()              |
         arctanh()           |
         ...         
        _____________________________________________________________________________________________
         conj()              |
         conjugate()         | 
         ...
        _____________________________________________________________________________________________
         where()             |
         nonzero()           |
         all()               | 
         any()               | 
         ...
        _____________________________________________________________________________________________

 
 Linear Algebra Submodule:
 ==============================
    Linear Algebra algorithm implementations of the following functions are available
    in the submodule:
    np.linalg
    
        _____________________________________________________________________________________________
         Function            | Action
        _____________________________________________________________________________________________
         norm()              |
         rank()              |
         eig()               |
         svd()               | 
         cholesky()          | 
         det()               | 
         inv()               | 
         pinv()              | 
         ...
        _____________________________________________________________________________________________
    
 
 Random Numbers:
 ==============================
    random number generators are available in the submodule 
    np.random
    
        _____________________________________________________________________________________________
         Function            | Action
        _____________________________________________________________________________________________
         rand()              |
         randint()           |
         randn()             |
         ...
        _____________________________________________________________________________________________
         seed                |
         get_state           |
         set_state           |         
        _____________________________________________________________________________________________
         uniform()           |
         normal()            | 
         standard_t()        | 
         binomial()          | 
         possion()           | 
         beta()              | 
         chisquare()         | 
         gamma()             | 
         weibull()           | 
         ...
        _____________________________________________________________________________________________
    
"""
print(np_remark)


raw_input("press Enter to continue")
# Exercise 0:
exercise1 = """
# -------------------------------------------------------------------------------------------------- #
#                                           Exercise 1:                                              #
#                                                                                                    #
# 1- Create a one dimensional float numpy array named array_2 containing numbers                     #
#    ranging from 3 to 55, with step 2 efficiently, and print it                                     #
#                                                                                                    #
# -------------------------------------------------------------------------------------------------- #
"""
print(exercise0)


# ================================================================================================== #

